/*
 * Copyright (c) 2003 Bernd Walter Computer Technology
 * http://www.bwct.de
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $URL: https://devel.bwct.de/svn/projects/modbus/usb/usblcd/freebsd/ulcd.c $
 * $Date: 2005-06-16 03:35:12 +0200 (Thu, 16 Jun 2005) $
 * $Author: ticso $
 * $Rev: 2157 $
 */

/*
 * ./ulcd -i -c 40 -c 14 -c 6
 * 
 * -i initialisiert das Display - genaugenommen passiert damit nichts und
 * ist fr zuknftige Erweiterungen gedacht.
 * 
 * -c zahl die folgende Zahl als Komando zum  Display.
 * 40 ist dabei 4bit Betrieb, 2 Zeilen, 5x8 Zeichmatrix.
 *    Der Converter betreibt das Display im 4 bit Modus, damit im Falle
 *    einer freien Verkabelung weniger Leitungen zu legen sind.
 * 14 ist Display an und nicht blinkender Cursor.
 * 6 ist Cursor nach rechts bewegen - Displayinhalt nicht bewegen.
 * 
 * Im Detail im HD44780 Datenblatt nachzulesen.
 * Datenblatt als PDF ist unter http://www.bwct.de/hd44780.pdf hinterlegt.
 * Die Tabelle der Komandos und Beschreibungen stehen ab Seite 191.
 * 
 * ./ulcd -c 128 -s "USB LCD V1.0    "
 * 
 * -c 128 setzt die Cursorposition auf die 1. Zeile 1. Zeichen.
 * -s gibt den folgenden String ab der Cursorposition aus.
 * 
 * ./ulcd -c 192 -s "(c) 2004 by BWCT"
 * 
 * -c 192 setzt die Cursorposition auf die 2. Zeile 1. Zeichen.
 * -s gibt den folgenden String ab der Cursorposition aus.
 * 
 * ./ulcd -b 30
 * 
 * -b setzt den Kontrast entsprechend der nachfolgenden Zahl.
 */

#include <sys/types.h>
#include <sys/ioctl.h>
#include <dev/usb/usb.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "ulcd.h"

int main(int argc, char *argv[]);
void usage(void);

int
main(int argc, char *argv[]) {
	struct usb_ctl_request request;
	int res;
	int lcd;
	char* tmp;
	char ch;
	int port;
	
	if (argc < 2)
		usage();

	lcd = open(argv[1], O_RDWR | O_EXCL);
	if (lcd < 0) {
		printf("opening device %s failed\n", argv[1]);
		exit(1);
	}
	argc -= 1;
	argv += 1;

	port = 0;
	while ((ch = getopt(argc, argv, "ib:c:d:s:p:")) != -1)
		switch (ch) {
		case 'i':	/* init LCD */
			bzero(&request, sizeof(request));
			request.ucr_request.wIndex[0] = port;
			request.ucr_request.bmRequestType = UT_VENDOR;
			request.ucr_request.bRequest = VENDOR_LCD_RESET;
			res = ioctl(lcd, USB_DO_REQUEST, &request);
			if (res < 0) {
				printf("USB request failed: %s\n",
				    strerror(errno));
				exit(1);
			}
			break;
		case 'b':	/* set contrast */
			bzero(&request, sizeof(request));
			request.ucr_request.wIndex[0] = port;
			request.ucr_request.bmRequestType = UT_VENDOR;
			request.ucr_request.bRequest = VENDOR_LCD_CONTRAST;
			request.ucr_request.wValue[0] = atol(optarg);
			res = ioctl(lcd, USB_DO_REQUEST, &request);
			if (res < 0) {
				printf("USB request failed: %s\n",
				    strerror(errno));
				exit(1);
			}
			break;
		case 'c':	/* command byte */
			bzero(&request, sizeof(request));
			request.ucr_request.wIndex[0] = port;
			request.ucr_request.bmRequestType = UT_VENDOR;
			request.ucr_request.bRequest = VENDOR_LCD_CMD;
			request.ucr_request.wValue[0] = atol(optarg);
			res = ioctl(lcd, USB_DO_REQUEST, &request);
			if (res < 0) {
				printf("USB request failed: %s\n",
				    strerror(errno));
				exit(1);
			}
			break;
		case 'd':	/* data byte */
			bzero(&request, sizeof(request));
			request.ucr_request.wIndex[0] = port;
			request.ucr_request.bmRequestType = UT_VENDOR;
			request.ucr_request.bRequest = VENDOR_LCD_DATA;
			request.ucr_request.wValue[0] = atol(optarg);
			res = ioctl(lcd, USB_DO_REQUEST, &request);
			if (res < 0) {
				printf("USB request failed: %s\n",
				    strerror(errno));
				exit(1);
			}
			break;
		case 's':	/* string of data bytes */
			bzero(&request, sizeof(request));
			request.ucr_request.wIndex[0] = port;
			request.ucr_request.bmRequestType = UT_VENDOR;
			request.ucr_request.bRequest = VENDOR_LCD_DATA;
			for (tmp = optarg; *tmp != '\0'; tmp++) {
				request.ucr_request.wValue[0] = *tmp;
				res = ioctl(lcd, USB_DO_REQUEST, &request);
				if (res < 0) {
					printf("USB request failed: %s\n",
					    strerror(errno));
					exit(1);
				}
			}
			break;
		case 'p':	/* setup port */
			port = atol(optarg);
			break;
		default:
			usage();
			/* NOTREACHED */
		}
	argc -= optind;
	argv += optind;

	close(lcd);
	return (0);
}

void
usage(void) {

	printf("usage: ulcd dev [args]\n");
	exit(1);
}

